searchNodes=[{"doc":"Loader is a load-generating library that allows you to define arbitrary distributions of arbitrary work via mathematical functions and small structs. These distributions, called LoadProfile s, can be paired up with a WorkSpec and executed to generate load, and gather statistics from a client's perspective. Example Let's assume there is some service at http://website.io/public/api , and we want to generate some simple, uniform load against that service. alias Loader . { LoadProfile , WorkResponse , WorkSpec } uniform_one_minute_profile = LoadProfile . new ( %{ target_running_time : 60 , function : &amp; LoadProfile.Curves . uniform ( &amp;1 , 10 ) # y = 10 } ) service_call_spec = % WorkSpec { task : fn -&gt; Finch . build ( :get , &quot;http://website.io/public/api&quot; , [ ] ) |&gt; Finch . request ( MyApp.Finch ) end , is_success? : fn % WorkResponse { data : res } -&gt; case res do { :ok , _any } -&gt; true _any -&gt; false end end } Loader . execute_profile ( uniform_one_minute_profile , service_call_spec ) The above example will generate a uniform 10 requests/ second against our imaginary service. We could also write additional load profiles, and run them concurrently to generate constructive interference! linear_one_minute_profile = LoadProfile . new ( %{ target_running_time : 60 , function : &amp; LoadProfile.Curves . linear ( &amp;1 , 1.5 , 5 ) # y = 1.5x + 5 } ) sine_wave_one_minute_profile = LoadProfile . new ( %{ target_running_time : 60 , function : fn x -&gt; 5 * ( :math . sin ( x ) + 1 ) end # y = 5 * (sin(x) + 1) } ) Loader . execute_profiles ( [ { uniform_one_minute_profile , service_call_spec } , { linear_one_minute_profile , service_call_spec } , { sine_wave_one_minute_profile , service_call_spec } , ] ) Visualized, this second example would produce load on the service as shown, where x is in seconds and y is requests/ second: Telemetry Loader emits the following telemetry events: [:loader, :load_profile_execution, :start] - emitted when Loader.execute_profile/2 or Loader.execute_profiles/1 is called. [:loader, :load_profile_execution, :stop] - emitted when a LoadProfile has been fully emitted, regardless of the number of successes or failures of individual tasks [:loader, :task, :start] - emitted when the :task callback from a Loader.WorkSpec is invoked [:loader, :task, :stop] - emitted when the :task callback from a Loader.WorkSpec is invoked [:loader, :task, :exception] - emitted if there is an uncaught exception while invoking the :task callback from a Loader.WorkSpec See the documentation for the Loader.Telemetry module for more information. Installation The package can be installed by adding loader to your list of dependencies in mix.exs : def deps do [ { :loader , &quot;~&gt; 0.2.0&quot; } ] end The docs can be found at https://hexdocs.pm/loader .","ref":"Loader.html","title":"Loader","type":"module"},{"doc":"Execute tasks based on the work_spec , scheduled based on the parameters in the load_profile","ref":"Loader.html#execute_profile/2","title":"Loader.execute_profile/2","type":"function"},{"doc":"Execute several LoadProfile s simultaneously, each with their supplied WorkSpec","ref":"Loader.html#execute_profiles/1","title":"Loader.execute_profiles/1","type":"function"},{"doc":"All entries in the ets tables &quot;held&quot; by this process should have the following format: TODO: add duration after mono completion time { unique_ref , # via `make_ref/0` pid , monotomic_start_time , successful_monotomic_completion_time , wall_clock_start_time , successful_wall_clock_completion_time , total_tasks_to_execute , success_count , failure_count } The details of the ets table and the tuple format are an internal detail, and should not be relied upon by other modules.","ref":"Loader.ExecutionStore.html","title":"Loader.ExecutionStore","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Loader.ExecutionStore.html#child_spec/1","title":"Loader.ExecutionStore.child_spec/1","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#increment_failures/1","title":"Loader.ExecutionStore.increment_failures/1","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#increment_successes/1","title":"Loader.ExecutionStore.increment_successes/1","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#log_successful_termination/1","title":"Loader.ExecutionStore.log_successful_termination/1","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#new_scheduled_loader/2","title":"Loader.ExecutionStore.new_scheduled_loader/2","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#start_link/1","title":"Loader.ExecutionStore.start_link/1","type":"function"},{"doc":"","ref":"Loader.ExecutionStore.html#task_counts/1","title":"Loader.ExecutionStore.task_counts/1","type":"function"},{"doc":"A struct representing a distribution of discrete work tasks over a period of time, and functions for working with that data. See Loader.LoadProfile.Curves for details on defining the function of a LoadProfile , which determines the distribution of work tasks. A LoadProfile is defined independently from the type of work being done. It could describe calls made against a remote service as easily as work done in a local module.","ref":"Loader.LoadProfile.html","title":"Loader.LoadProfile","type":"module"},{"doc":"Returns a new LoadProfile based on the given props, with incorrect props set to default values. Rules for props: target_running_time : must be a positive integer function : must be a 1-arity function. It should also return a number, but this isn't enforced","ref":"Loader.LoadProfile.html#new/1","title":"Loader.LoadProfile.new/1","type":"function"},{"doc":"Returns a 2-tuple: a plot of points that represents how tasks would be distributed for the given profile, and the total number of tasks &quot;under the curve&quot; (an approximate integral of the function, reflecting the total number of tasks that will be executed).","ref":"Loader.LoadProfile.html#plot_curve/1","title":"Loader.LoadProfile.plot_curve/1","type":"function"},{"doc":"","ref":"Loader.LoadProfile.html#t:t/0","title":"Loader.LoadProfile.t/0","type":"type"},{"doc":"Convenience functions for defining typical &quot;curves&quot; (i.e. &quot;functions&quot;) for the request distribution in a LoadProfile . Some function families, like exponentiation and logarithms, are varied enough or complex enough that i believe they are better expressed as &quot;plain&quot; callback functions, e.g. fn x -&gt; -1 * :math.pow(2, (x + 3)) + 4 . These function families have thus been excluded from this module. In the context of a LoadProfile , the unit for x is always seconds .","ref":"Loader.LoadProfile.Curves.html","title":"Loader.LoadProfile.Curves","type":"module"},{"doc":"","ref":"Loader.LoadProfile.Curves.html#linear/3","title":"Loader.LoadProfile.Curves.linear/3","type":"function"},{"doc":"","ref":"Loader.LoadProfile.Curves.html#quadratic/4","title":"Loader.LoadProfile.Curves.quadratic/4","type":"function"},{"doc":"A sinusoidal function with a modification such that the result is always &gt;= 0. Accepts options to modify the oscillation of the wave. Options :amplitude : a measure of the peak deviation of the wave from it's center. To keep all values of the wave &gt;= 0, the vertical center of the wave will also be equal to its amplitude. Should be a positive number, and will be forced as such via Kernel.abs/1 . Defaults to 1 . :frequency : the number of oscillations (cycles) that occur each second. Defaults to 1 . :angular_frequency : the rate-of-change of the function, in units of radians/second. Mutually exclusive with :frequency , with :angular_frequency taking precedence. Defaults to nil . :phase : specifies, in radians, where in the wave's cycle the oscillation will begin, when x = 0. Defaults to 0 . See https://en.wikipedia.org/wiki/Sine_wave for more info on sine waves","ref":"Loader.LoadProfile.Curves.html#sine_wave/2","title":"Loader.LoadProfile.Curves.sine_wave/2","type":"function"},{"doc":"","ref":"Loader.LoadProfile.Curves.html#uniform/2","title":"Loader.LoadProfile.Curves.uniform/2","type":"function"},{"doc":"Telemetry integration. Unless specified, all times are in :native units. Loader executes the following events: Load Profile Execution Start [:loader, :load_profile_execution, :start] - emitted when Loader.execute_profile/2 or Loader.execute_profiles/1 is called. Measurements :system_time - system time, as reported by System.system_time/0 Metadata :load_profile - profile ( Loader.LoadProfile ) :scheduled_loader_ref - (almost) unique reference generated for the process that is handling execution of the profile. Generated with make_ref/0 :work_spec - specification for each task to be executed ( Loader.WorkSpec ) Load Profile Execution Stop [:loader, :load_profile_execution, :stop] - emitted when a LoadProfile has been fully executed, regardless of the number of successes or failures of individual tasks Measurements :duration - time elapsed since the load profile execution start event Metadata :failures - number of tasks deemed unsuccessful, based on the WorkSpec :load_profile - profile ( Loader.LoadProfile ) :scheduled_loader_ref - (almost) unique reference generated for the process that handled execution of the profile. Generated with make_ref/0 :successes - number of tasks deemed successful, based on the WorkSpec :work_spec - specification for each task to be executed ( Loader.WorkSpec ) Task Start [:loader, :task, :start] - emitted when the :task callback from a Loader.WorkSpec is invoked Measurements :monotonic_time - system monotonic time, as reported by :erlang.monotonic_time/0 (see the :telemetry docs span/3 function for more details) :system_time - system time, as reported by :erlang.system_time/0 (see the :telemetry docs span/3 function for more details) Metadata :scheduled_loader_ref - (almost) unique reference generated for the process that handled execution of the profile. Generated with make_ref/0 :telemetry_span_context - (almost) unique reference generated for the span execution :work_spec - specification for the task ( Loader.WorkSpec ) Task Stop [:loader, :task, :stop] - emitted when the :task callback from a Loader.WorkSpec is invoked Measurements :monotonic_time - system monotonic time, as reported by :erlang.monotonic_time/0 (see the :telemetry docs span/3 function for more details) :system_time - system time, as reported by :erlang.system_time/0 (see the :telemetry docs span/3 function for more details) Metadata :scheduled_loader_ref - (almost) unique reference generated for the process that handled execution of the profile. Generated with make_ref/0 :telemetry_span_context - (almost) unique reference generated for the span execution :was_success? - whether or not the task was successful, according to the WorkSpec :work_spec - specification for the task ( Loader.WorkSpec ) Task Exception [:loader, :task, :exception] - emitted if there is an uncaught exception while invoking the :task callback from a Loader.WorkSpec Measurements :monotonic_time - system monotonic time, as reported by :erlang.monotonic_time/0 (see the :telemetry docs span/3 function for more details) :system_time - system time, as reported by :erlang.system_time/0 (see the :telemetry docs span/3 function for more details) Metadata :kind - the type of exception :reason - whatever reason is given by the exception :scheduled_loader_ref - (almost) unique reference generated for the process that handled execution of the profile. Generated with make_ref/0 :stacktrace - the stacktrace of the exception :telemetry_span_context - (almost) unique reference generated for the span execution :work_spec - specification for the task ( Loader.WorkSpec )","ref":"Loader.Telemetry.html","title":"Loader.Telemetry","type":"module"},{"doc":"Internal data structure used to represent the results of executing a WorkSpec Properties :data - whatever important data is returned by the work :response_time - must be an integer number, in microseconds , which is the &quot;client-side&quot; view of how long the work took. I recommend using System.monotonic_time/0 or :timer.tc/1","ref":"Loader.WorkResponse.html","title":"Loader.WorkResponse","type":"module"},{"doc":"","ref":"Loader.WorkResponse.html#t:t/0","title":"Loader.WorkResponse.t/0","type":"type"},{"doc":"A specification for some &quot;work&quot; to do, to generate load.","ref":"Loader.WorkSpec.html","title":"Loader.WorkSpec","type":"module"},{"doc":"","ref":"Loader.WorkSpec.html#t:t/0","title":"Loader.WorkSpec.t/0","type":"type"}]